package eu.jonahbauer.json.query.parser.ast;

import eu.jonahbauer.json.JsonValue;
import eu.jonahbauer.json.query.impl.Generator;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.NoSuchElementException;

public record JQForEachExpression(
        @NotNull JQExpression expression,
        @NotNull JQPatterns patterns,
        @NotNull JQExpression init,
        @NotNull JQExpression update,
        @NotNull JQExpression extract
) implements JQExpression {

    @Override
    public @NotNull Generator<@Nullable JsonValue> evaluate(@NotNull Context context) {
        return init.evaluate(context).flatMap(initial -> {
            var state = new Object() {
                private JsonValue state = initial;
            };
            return expression.evaluate(context).flatMap(value -> patterns.bind(context, value, ctx -> {
                return update.evaluate(ctx.withRoot(state.state)).map(v -> {
                    state.state = v;
                    return ctx.withRoot(v);
                }).flatMap(extract::evaluate);
            }));
        });
    }

    @Override
    public boolean isConstant() {
        return expression.isConstant() && init.isConstant() && update.isConstant();
    }

    @Override
    public @NotNull String toString() {
        return "foreach " + expression + " as " + patterns + " (" + init + "; " + update + "; " + extract + ")";
    }
}
