package eu.jonahbauer.json.query.parser.ast;

import eu.jonahbauer.json.JsonObject;
import eu.jonahbauer.json.JsonString;
import eu.jonahbauer.json.JsonValue;
import eu.jonahbauer.json.query.JsonQueryException;
import eu.jonahbauer.json.query.impl.Generator;
import eu.jonahbauer.json.query.util.Util;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.StringJoiner;

public record JQObjectConstructionExpression(@NotNull List<@NotNull Entry> entries) implements JQExpression {

    @Override
    public @NotNull Generator<@Nullable JsonValue> evaluate(@NotNull Context context) {
        var generator = Generator.of(new LinkedHashMap<@NotNull String, @Nullable JsonValue>());
        for (var entry : entries) {
            generator = generator.flatMap(map -> entry.key().evaluate(context)
                    .flatMap(key -> {
                        if (!(key instanceof JsonString(var string))) {
                            throw new JsonQueryException("Cannot use " + Util.type(key) + "(" + Util.value(key) + ") as object key.");
                        }
                        return entry.value().evaluate(context)
                                .map(value -> {
                                    var out = new LinkedHashMap<>(map);
                                    out.put(string, value);
                                    return out;
                                });
                    })
            );
        }
        return generator.map(JsonObject::new);
    }

    @Override
    public boolean isConstant() {
        return false;
    }

    @Override
    public @NotNull String toString() {
        var out = new StringJoiner(", ", "{", "}");
        entries.forEach(entry -> out.add(entry.key() + ": " + entry.value()));
        return out.toString();
    }

    public record Entry(@NotNull JQExpression key, @NotNull JQExpression value) { }
}
