package eu.jonahbauer.json.query.parser.ast;

import eu.jonahbauer.json.JsonValue;
import eu.jonahbauer.json.query.JsonMath;
import eu.jonahbauer.json.query.impl.Generator;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Objects;

public record JQSliceExpression(@NotNull JQExpression expression, @Nullable JQExpression start, @Nullable JQExpression end, boolean optional) implements JQExpression {
    public JQSliceExpression {
        Objects.requireNonNull(expression);
        if (start == null && end == null) throw new IllegalArgumentException();
    }

    @Override
    public @NotNull Generator<@Nullable JsonValue> evaluate(@NotNull Context context) {
        return expression.evaluate(context)
                .flatMap(value -> (start == null ? Generator.of((JsonValue) null) : start.evaluate(context))
                        .flatMap(start -> (end == null ? Generator.of((JsonValue) null) : end.evaluate(context))
                                .map(end -> JsonMath.slice(value, start, end))
                        )
                );
    }

    @Override
    public boolean isConstant() {
        return expression.isConstant() && (start == null || start.isConstant()) && (end == null || end.isConstant());
    }

    @Override
    public @NotNull String toString() {
        return expression + "[" + (start == null ? "" : start) + ":" + (end == null ? "" : end) + "]";
    }
}
